@@ -273,6 +273,7 @@ typedef enum {
   tagPLAINTEXT,
   tagBLINK,
   tagMARQUEE,
+  tagNLMARK,      /* New Line Code */
 } tag_type;
 
 typedef struct mod_chxj_config mod_chxj_config;
@@ -345,6 +346,8 @@ struct mod_chxj_config {
 #if defined(USE_MEMCACHE_COOKIE)
   memcache_t            memcache;
 #endif
+  char                  *forward_url_base;  /* use input filter */
+  char                  *forward_server_ip; /* use input filter */
 
   chxj_new_line_type_t  new_line_type;
 };
@@ -399,6 +402,7 @@ typedef struct {
 #define HTTP_USER_AGENT       "User-Agent"
 #define HTTP_HOST             "Host"
 #define CHXJ_HTTP_USER_AGENT  "CHXJ_HTTP_USER_AGENT"
+#define CHXJ_HEADER_ORIG_CLIENT_IP "X-Chxj-Orig-Client-Ip"
 
 module AP_MODULE_DECLARE_DATA chxj_module;
 
@@ -437,6 +441,7 @@ extern char* chxj_node_convert(
 
 #define IMAGE_CACHE_LIMIT_FMT_LEN  (20)
 
+
 #if HAVE_MALLOC == 0
 extern void *rpl_malloc(size_t n);
 #endif
