@@ -116,6 +116,7 @@ static char *s_jhtml_start_blink_tag  (void *pdoc, Node *node);
 static char *s_jhtml_end_blink_tag    (void *pdoc, Node *node);
 static char *s_jhtml_start_marquee_tag(void *pdoc, Node *node);
 static char *s_jhtml_end_marquee_tag  (void *pdoc, Node *node);
+static char *s_jhtml_newline_mark       (void *pdoc, Node *node);
 
 static void  s_init_jhtml(jhtml_t *jhtml, Doc *doc, request_rec *r, device_table *spec);
 
@@ -391,6 +392,11 @@ tag_handler jhtml_handler[] = {
     s_jhtml_start_marquee_tag,
     s_jhtml_end_marquee_tag,
   },
+  /* tagNLMARK */
+  {
+    s_jhtml_newline_mark,
+    NULL,
+  },
 };
 
 
@@ -418,6 +424,7 @@ chxj_convert_jhtml(
   char      *ss;
   jhtml_t   jhtml;
   Doc       doc;
+  apr_pool_t *pool;
 
   dst = NULL;
 
@@ -448,7 +455,8 @@ chxj_convert_jhtml(
   qs_init_malloc(&doc);
   qs_init_root_node(&doc);
 
-  ss = apr_pcalloc(r->pool, srclen + 1);
+  apr_pool_create(&pool, r->pool);
+  ss = apr_pcalloc(pool, srclen + 1);
 
   memset(ss,   0, srclen + 1);
   memcpy(ss, src, srclen);
@@ -465,17 +473,17 @@ chxj_convert_jhtml(
   /*--------------------------------------------------------------------------*/
   chxj_node_convert(spec,r,(void*)&jhtml, &doc, qs_get_root(&doc), 0);
   jhtml.out = chxj_buffered_write_flush(jhtml.out, &doc.buf);
-  dst = apr_pstrdup(r->pool, jhtml.out);
+  dst = apr_pstrdup(pool, jhtml.out);
   chxj_buffered_write_terminate(&doc.buf);
 
 
   qs_all_free(&doc,QX_LOGMARK);
 
   if (! dst) 
-    return apr_pstrdup(r->pool,ss);
+    return apr_pstrdup(pool,ss);
 
   if (! strlen(dst)) 
-    dst = apr_psprintf(r->pool, "\n");
+    dst = apr_psprintf(pool, "\n");
 
   *dstlen = strlen(dst);
 
@@ -605,7 +613,6 @@ s_jhtml_start_html_tag(void *pdoc, Node *UNUSED(node))
   /* start HTML tag                                                           */
   /*--------------------------------------------------------------------------*/
   W_L("<html>");
-  W_NLCODE();
 
   DBG(r, "end s_jhtml_start_html_tag()");
 
@@ -628,7 +635,6 @@ s_jhtml_end_html_tag(void *pdoc, Node *UNUSED(child))
   Doc           *doc = jhtml->doc;
 
   W_L("</html>");
-  W_NLCODE();
 
   return jhtml->out;
 }
@@ -738,7 +744,6 @@ s_jhtml_start_meta_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -780,7 +785,6 @@ s_jhtml_start_head_tag(void *pdoc, Node *UNUSED(node))
   r     = doc->r;
 
   W_L("<head>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -805,7 +809,6 @@ s_jhtml_end_head_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</head>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -854,7 +857,6 @@ s_jhtml_end_title_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</title>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -895,7 +897,6 @@ s_jhtml_start_base_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -983,7 +984,6 @@ s_jhtml_start_body_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1008,7 +1008,6 @@ s_jhtml_end_body_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</body>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1157,7 +1156,6 @@ s_jhtml_end_a_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</a>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1200,7 +1198,6 @@ s_jhtml_start_br_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1241,7 +1238,6 @@ s_jhtml_start_tr_tag(void *pdoc, Node *UNUSED(node))
   r     = doc->r;
 
   W_L("<br>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1333,7 +1329,6 @@ s_jhtml_end_font_tag(void *pdoc, Node *UNUSED(child))
 
   if (jhtml->font_flag) {
     W_L("</font>");
-    W_NLCODE();
     jhtml->font_flag--;
   }
   return jhtml->out;
@@ -1355,8 +1350,6 @@ s_jhtml_start_form_tag(void *pdoc, Node *node)
   Doc          *doc;
   request_rec  *r;
   Attr         *attr;
-  int          dcflag = 0;
-  char         *dc = NULL;
 
   jhtml = GET_JHTML(pdoc);
   doc   = jhtml->doc;
@@ -1375,13 +1368,10 @@ s_jhtml_start_form_tag(void *pdoc, Node *node)
       /*----------------------------------------------------------------------*/
       /* CHTML 1.0                                                            */
       /*----------------------------------------------------------------------*/
+      value = chxj_add_cookie_parameter(r, value, jhtml->cookie);
       W_L(" action=\"");
       W_V(value);
       W_L("\"");
-      dc = chxj_add_cookie_parameter(r, value, jhtml->cookie);
-      if (strcmp(dc, value)) {
-        dcflag = 1;
-      } 
     }
     else if (STRCASEEQ('m','M',"method",name)) {
       /*----------------------------------------------------------------------*/
@@ -1400,23 +1390,6 @@ s_jhtml_start_form_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  /*-------------------------------------------------------------------------*/
-  /* ``action=""''                                                           */
-  /*-------------------------------------------------------------------------*/
-  if (! dc) {
-    dcflag = 1;
-  }
-  /*-------------------------------------------------------------------------*/
-  /* Add cookie parameter                                                    */
-  /*-------------------------------------------------------------------------*/
-  if (jhtml->cookie && jhtml->cookie->cookie_id && dcflag == 1) {
-    char *vv = apr_psprintf(doc->buf.pool, "%s<input type='hidden' name='%s' value='%s'>",
-                            jhtml->out, 
-                            CHXJ_COOKIE_PARAM,
-                            chxj_url_decode(doc->buf.pool, jhtml->cookie->cookie_id));
-    W_V(vv);
-  }
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1435,7 +1408,6 @@ s_jhtml_end_form_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc   = jhtml->doc;
   W_L("</form>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1513,7 +1485,7 @@ s_jhtml_start_input_tag(void *pdoc, Node *node)
   }
   if (value && *value) {
     W_L(" value=\"");
-    W_V(value);
+    W_V(chxj_add_slash_to_doublequote(doc->pool, value));
     W_L("\"");
   }
   if (accesskey && *accesskey) {
@@ -1557,7 +1529,6 @@ s_jhtml_start_input_tag(void *pdoc, Node *node)
     W_L(" checked");
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1592,7 +1563,6 @@ s_jhtml_start_center_tag(void *pdoc, Node *UNUSED(node))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc   = jhtml->doc;
   W_L("<center>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1617,7 +1587,6 @@ s_jhtml_end_center_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</center>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1732,7 +1701,6 @@ s_jhtml_start_ol_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1757,7 +1725,6 @@ s_jhtml_end_ol_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</ol>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1805,7 +1772,6 @@ s_jhtml_start_p_tag(void *pdoc, Node *node)
     W_L("\"");
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1825,7 +1791,6 @@ s_jhtml_end_p_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc   = jhtml->doc;
 
   W_L("</p>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1846,7 +1811,6 @@ s_jhtml_start_pre_tag(void *pdoc, Node *UNUSED(node))
 
   jhtml->pre_flag++;
   W_L("<pre>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1866,7 +1830,6 @@ s_jhtml_end_pre_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = jhtml->doc;
 
   W_L("</pre>");
-  W_NLCODE();
   jhtml->pre_flag--;
 
   return jhtml->out;
@@ -1888,7 +1851,6 @@ s_jhtml_start_ul_tag(void *pdoc, Node *UNUSED(node))
   Doc     *doc   = jhtml->doc;
 
   W_L("<ul>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1908,7 +1870,6 @@ s_jhtml_end_ul_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = jhtml->doc;
 
   W_L("</ul>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1976,7 +1937,6 @@ s_jhtml_start_hr_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2117,7 +2077,6 @@ s_jhtml_start_img_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2195,7 +2154,6 @@ s_jhtml_start_select_tag(void *pdoc, Node *child)
     W_L(" multiple");
   }
   W_L(">");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2215,7 +2173,6 @@ s_jhtml_end_select_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = jhtml->doc;
 
   W_L("</select>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2352,7 +2309,6 @@ s_jhtml_end_div_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</div>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2482,7 +2438,6 @@ s_jhtml_end_textarea_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</textarea>");
-  W_NLCODE();
   jhtml->textarea_flag--;
 
   return jhtml->out;
@@ -2528,7 +2483,6 @@ s_jhtml_end_b_tag(void* pdoc, Node* UNUSED(child))
   Doc*          doc   = jhtml->doc;
 
   W_L("</b>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2548,6 +2502,7 @@ s_jhtml_text_tag(void* pdoc, Node* child)
   jhtml = GET_JHTML(pdoc);
   doc   = jhtml->doc;
   r     = doc->r;
+  DBG(r, "start s_jhtml_text_tag()");
 
   textval = qs_get_node_value(doc,child);
   if (strlen(textval) == 0) {
@@ -2595,6 +2550,7 @@ s_jhtml_text_tag(void* pdoc, Node* child)
     }
   }
   W_V(tdst);
+  DBG(r, "end s_jhtml_text_tag()");
   return jhtml->out;
 }
 
@@ -2613,7 +2569,6 @@ s_jhtml_start_blockquote_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc   = jhtml->doc;
   W_L("<blockquote>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2632,7 +2587,6 @@ s_jhtml_end_blockquote_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc   = jhtml->doc;
   W_L("</blockquote>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2651,7 +2605,6 @@ s_jhtml_start_dir_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("<dir>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2670,7 +2623,6 @@ s_jhtml_end_dir_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("</dir>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2689,7 +2641,6 @@ s_jhtml_start_dl_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("<dl>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2708,7 +2659,6 @@ s_jhtml_end_dl_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("</dl>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2846,7 +2796,6 @@ s_jhtml_end_h1_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h1_align_flag) {
     jhtml->h1_align_flag--;
     W_L("</div>");
-    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -2919,7 +2868,6 @@ s_jhtml_end_h2_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h2_align_flag) {
     jhtml->h2_align_flag--;
     W_L("</div>");
-    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -2992,7 +2940,6 @@ s_jhtml_end_h3_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h3_align_flag) {
     jhtml->h3_align_flag--;
     W_L("</div>");
-    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -3063,7 +3010,6 @@ s_jhtml_end_h4_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h4_align_flag) {
     jhtml->h4_align_flag--;
     W_L("</div>");
-    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -3134,7 +3080,6 @@ s_jhtml_end_h5_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h5_align_flag) {
     jhtml->h5_align_flag--;
     W_L("</div>");
-    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -3205,7 +3150,6 @@ s_jhtml_end_h6_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h6_align_flag) {
     jhtml->h6_align_flag--;
     W_L("</div>");
-    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -3225,7 +3169,6 @@ s_jhtml_start_menu_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc = jhtml->doc;
   W_L("<menu>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -3244,7 +3187,6 @@ s_jhtml_end_menu_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc = jhtml->doc;
   W_L("</menu>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -3336,7 +3278,6 @@ s_jhtml_end_blink_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc = jhtml->doc;
   W_L("</blink>");
-  W_NLCODE();
   return jhtml->out;
 }
 
@@ -3397,6 +3338,18 @@ s_jhtml_end_marquee_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("</marquee>");
+  return jhtml->out;
+}
+
+
+/**
+ * It is handler who processes the New Line Code.
+ */
+static char *
+s_jhtml_newline_mark(void *pdoc, Node *UNUSED(node))
+{
+  jhtml_t *jhtml = GET_JHTML(pdoc);
+  Doc *doc = jhtml->doc;
   W_NLCODE();
   return jhtml->out;
 }
