@@ -25,7 +25,6 @@ struct __app_ctx_t {
   int                 ssl_flag;
   serf_ssl_context_t  *ssl_ctx;
   serf_bucket_alloc_t *bkt_alloc;
-  request_rec *r;
 };
 
 struct __handler_ctx_t {
@@ -72,7 +71,6 @@ s_init(apr_pool_t *ppool, apr_pool_t **pool)
 
 
 
-
 static serf_bucket_t *
 s_connection_setup(apr_socket_t *skt, void *setup_ctx, apr_pool_t *UNUSED(pool))
 {
@@ -84,9 +82,8 @@ s_connection_setup(apr_socket_t *skt, void *setup_ctx, apr_pool_t *UNUSED(pool))
     c = serf_bucket_ssl_decrypt_create(c, ctx->ssl_ctx, ctx->bkt_alloc);
     if (!ctx->ssl_ctx) {
       ctx->ssl_ctx = serf_bucket_ssl_decrypt_context_get(c);
-      serf_ssl_use_default_certificates(ctx->ssl_ctx);
-      serf_ssl_server_cert_callback_set(ctx->ssl_ctx, NULL, NULL);
     }
+    return c;
   }
   return c;
 }
@@ -100,23 +97,19 @@ s_connection_closed(serf_connection_t *UNUSED(conn), void *UNUSED(closed_baton),
 
 
 static serf_bucket_t *
-s_accept_response(serf_request_t *request, serf_bucket_t *stream, void *acceptor_baton, apr_pool_t *UNUSED(pool))
+s_accept_response(serf_request_t *request, serf_bucket_t *stream, void *UNUSED(acceptor_baton), apr_pool_t *UNUSED(pool))
 {
     serf_bucket_alloc_t *bkt_alloc;
     serf_bucket_t       *c;
-    app_ctx_t           *ctx = acceptor_baton;
-    serf_bucket_t       *ret;
-    DBG(ctx->r, "REQ:[%x] start s_accept_response()", (unsigned int)(apr_size_t)ctx->r);
+
     bkt_alloc = serf_request_get_alloc(request);
     c = serf_bucket_barrier_create(stream, bkt_alloc);
-    ret = serf_bucket_response_create(c, bkt_alloc);
-    DBG(ctx->r, "REQ:[%x] end s_accept_response()", (unsigned int)(apr_size_t)ctx->r);
-    return ret;
+    return serf_bucket_response_create(c, bkt_alloc);
 }
 
 
 static apr_status_t 
-s_handle_response(serf_request_t *UNUSED(request), serf_bucket_t *response, void *handler_ctx, apr_pool_t *UNUSED(pool))
+s_handle_response(serf_request_t *UNUSED(request), serf_bucket_t *response, void *handler_ctx, apr_pool_t *pool)
 {
   const char      *data;
   apr_size_t      len;
@@ -124,72 +117,48 @@ s_handle_response(serf_request_t *UNUSED(request), serf_bucket_t *response, void
   apr_status_t     rv;
   handler_ctx_t  *ctx = handler_ctx;
 
-  DBG(ctx->r, "REQ[%x] start s_handle_response()", (unsigned int)(apr_size_t)ctx->r);
-
   rv = serf_bucket_response_status(response, &sl);
   if (rv != APR_SUCCESS) {
     if (APR_STATUS_IS_EAGAIN(rv)) {
-      DBG(ctx->r, "REQ[%x] end s_handle_response()", (unsigned int)(apr_size_t)ctx->r);
       return rv;
     }
     ctx->rv = rv;
-    {
-      char buf[200];
-      ERR(ctx->r, "%s:%d Error running context: (%d) %s\n", __FILE__,__LINE__,rv, apr_strerror(rv, buf, sizeof(buf)));
-    }
     apr_atomic_dec32(&ctx->requests_outstanding); 
-    DBG(ctx->r, "REQ[%x] end s_handle_response()", (unsigned int)(apr_size_t)ctx->r);
     return rv;
   }
   ctx->reason = sl.reason;
 
   while (1) {
-    len = 0;
     rv = serf_bucket_read(response, 2048, &data, &len);
     if (SERF_BUCKET_READ_ERROR(rv)) {
       ctx->rv = rv;
-      {
-        char buf[200];
-        ERR(ctx->r, "%s:%d Error running context: (%d) %s\n", __FILE__,__LINE__,rv, apr_strerror(rv, buf, sizeof(buf)));
-      }
       apr_atomic_dec32(&ctx->requests_outstanding);
-      DBG(ctx->r, "REQ[%x] end s_handle_response()", (unsigned int)(apr_size_t)ctx->r);
-      return rv;
-    }
-    if (APR_STATUS_IS_EAGAIN(rv)) {
-      /* 0 byte return if EAGAIN returned. */
-      DBG(ctx->r, "REQ[%X] end of s_handle_response() (EAGAIN) len:[%d]", (unsigned int)(apr_size_t)ctx->r, (int)len);
       return rv;
     }
 
-    if (len > 0) {
-      if (! ctx->response) {
-        ctx->response = apr_palloc(ctx->pool, len + 1);
-        ctx->response[0] = 0;
-        ctx->response_len = 0;
-      }
-      else {
-        char *tmp = apr_palloc(ctx->pool, ctx->response_len);
-        memcpy(tmp, ctx->response, ctx->response_len);
-        ctx->response = apr_palloc(ctx->pool, ctx->response_len + len + 1);
-        memcpy(ctx->response, tmp, ctx->response_len);
-      }
-    
-      memcpy(&ctx->response[ctx->response_len], data, len);
-      ctx->response_len += len;
-      ctx->response[ctx->response_len] = 0;
+    if (! ctx->response) {
+      ctx->response = apr_palloc(pool, len);
+      ctx->response[0] = 0;
+      ctx->response_len = 0;
     }
+    else {
+      char *tmp = apr_palloc(pool, ctx->response_len);
+      memcpy(tmp, ctx->response, ctx->response_len);
+      ctx->response = apr_palloc(pool, ctx->response_len + len);
+      memcpy(ctx->response, tmp, ctx->response_len);
+    }
+    
+    memcpy(&ctx->response[ctx->response_len], data, len);
+    ctx->response_len += len;
     if (APR_STATUS_IS_EOF(rv)) {
       serf_bucket_t *hdrs;
       char *tmp_headers = "";
       hdrs = serf_bucket_response_get_headers(response);
       while (1) {
         rv = serf_bucket_read(hdrs, 2048, &data, &len);
-        if (SERF_BUCKET_READ_ERROR(rv)) {
-          DBG(ctx->r, "REQ[%x] end s_handle_response()", (unsigned int)(apr_size_t)ctx->r);
+        if (SERF_BUCKET_READ_ERROR(rv))
           return rv;
-        }
-        tmp_headers = apr_pstrcat(ctx->pool, tmp_headers, apr_psprintf(ctx->pool , "%.*s", (int)len, data), NULL);
+        tmp_headers = apr_pstrcat(ctx->pool, tmp_headers, apr_psprintf(ctx->pool , "%.*s", len, data), NULL);
         if (APR_STATUS_IS_EOF(rv)) {
           break;
         }
@@ -213,15 +182,20 @@ s_handle_response(serf_request_t *UNUSED(request), serf_bucket_t *response, void
           val++;
           key = qs_trim_string(ctx->pool, key);
           val = qs_trim_string(ctx->pool, val);
-          DBG(ctx->r, "REQ[%X] RESPONSE key:[%s], val:[%s]", (unsigned int)(apr_size_t)ctx->r, key, val);
+          DBG(ctx->r, "key:[%s], val:[%s]", key, val);
           apr_table_add(ctx->headers_out, key, val);
         }
       }
       ctx->rv = APR_SUCCESS;
       apr_atomic_dec32(&ctx->requests_outstanding);
-      DBG(ctx->r, "REQ[%X] end of s_handle_response()(NORMAL)",(unsigned int)(apr_size_t)ctx->r);
+      DBG(ctx->r, "end of s_handle_response()(NORMAL)");
       return APR_EOF;
     }
+
+    if (APR_STATUS_IS_EAGAIN(rv)) {
+      DBG(ctx->r, "end of s_handle_response() (EAGAIN)");
+      return rv;
+    }
   }
 }
 
@@ -253,15 +227,13 @@ s_setup_request(serf_request_t           *request,
   apr_table_entry_t  *hentryp = (apr_table_entry_t*)headers->elts;
   for (ii=headers->nelts-1; ii>=0; ii--) {
     serf_bucket_headers_setc(hdrs_bkt, hentryp[ii].key, hentryp[ii].val);
-    DBG(ctx->r, "REQ[%X] REQUEST key:[%s], val:[%s]", (unsigned int)(apr_size_t)ctx->r, hentryp[ii].key, hentryp[ii].val);
+    DBG(ctx->r, "key:[%s], val:[%s]", hentryp[ii].key, hentryp[ii].val);
   }
   if (ctx->post_data) {
     serf_bucket_headers_setc(hdrs_bkt, "X-Chxj-Forward", "Done");
-    serf_bucket_headers_setc(hdrs_bkt, "X-Chxj-Content-Length", apr_psprintf(r->pool, "%" APR_SIZE_T_FMT , ctx->post_data_len));
-    DBG(ctx->r, "REQ[%X] REQUEST key:[%s], val:[%s]", (unsigned int)(apr_size_t)ctx->r, "X-Chxj-Forward", "Done");
-    DBG(ctx->r, "REQ[%X] REQUEST key:[%s], val:[%s]", (unsigned int)(apr_size_t)ctx->r, "X-Chxj-Content-Length", apr_psprintf(r->pool, "%" APR_SIZE_T_FMT, ctx->post_data_len));
+    serf_bucket_headers_setc(hdrs_bkt, "X-Chxj-Content-Length", apr_psprintf(r->pool, "%d", ctx->post_data_len));
   }
-  DBG(ctx->r, "REQ[%X] REQUEST Content-Length:[%s]", (unsigned int)(apr_size_t)r, serf_bucket_headers_get(hdrs_bkt, "Content-Length"));
+  DBG(ctx->r, "Content-Length:[%s]", serf_bucket_headers_get(hdrs_bkt, "Content-Length"));
 
   apr_atomic_inc32(&(ctx->requests_outstanding));
   if (ctx->acceptor_ctx->ssl_flag) {
@@ -330,8 +302,6 @@ default_chxj_serf_get(request_rec *r, apr_pool_t *ppool, const char *url_path, i
   if (strcasecmp(url.scheme, "https") == 0) {
     app_ctx.ssl_flag = 1;
   }
-  app_ctx.r = r;
-  app_ctx.ssl_ctx = NULL;
 
   context = serf_context_create(pool);
   connection = serf_connection_create(context, address, s_connection_setup, &app_ctx, s_connection_closed, &app_ctx, pool);
@@ -420,9 +390,6 @@ default_chxj_serf_post(request_rec *r, apr_pool_t *ppool, const char *url_path,
   if (!url.hostname) {
     url.hostname = "localhost";
   }
-  if (url.query) {
-    url.path = apr_psprintf(pool, "%s?%s", url.path, url.query);
-  }
 
   rv = apr_sockaddr_info_get(&address, url.hostname, APR_UNSPEC, url.port, 0, pool);
   if (rv != APR_SUCCESS) {
@@ -436,8 +403,6 @@ default_chxj_serf_post(request_rec *r, apr_pool_t *ppool, const char *url_path,
   if (strcasecmp(url.scheme, "https") == 0) {
     app_ctx.ssl_flag = 1;
   }
-  app_ctx.r = r;
-  app_ctx.ssl_ctx = NULL;
 
   context = serf_context_create(pool);
   connection = serf_connection_create(context, address, s_connection_setup, &app_ctx, s_connection_closed, &app_ctx, pool);
@@ -467,27 +432,22 @@ default_chxj_serf_post(request_rec *r, apr_pool_t *ppool, const char *url_path,
       continue;
     if (rv) {
       char buf[200];
-      ERR(r, "%s:%d Error running context: (%d) %s", __FILE__,__LINE__,rv, apr_strerror(rv, buf, sizeof(buf)));
+      ERR(r, "Error running context: (%d) %s\n", rv, apr_strerror(rv, buf, sizeof(buf)));
       break;
     }
     if (!apr_atomic_read32(&handler_ctx.requests_outstanding)) {
       if (handler_ctx.rv != APR_SUCCESS) {
         char buf[200];
-        ERR(r, "%s:%d Error running context: (%d) %s", __FILE__,__LINE__,handler_ctx.rv, apr_strerror(handler_ctx.rv, buf, sizeof(buf)));
+        ERR(r, "Error running context: (%d) %s\n", handler_ctx.rv, apr_strerror(handler_ctx.rv, buf, sizeof(buf)));
       }
       break;
     }
   }
 
   DBG(r, "end of serf request");
-  DBG(r, "response:[%s][%" APR_SIZE_T_FMT "]", handler_ctx.response, handler_ctx.response_len);
+  DBG(r, "response:[%s][%d]", handler_ctx.response, handler_ctx.response_len);
   serf_connection_close(connection);
-  if (handler_ctx.response_len == 0) {
-    ret = apr_pstrdup(ppool, "");
-  }
-  else {
-    ret = apr_pstrdup(ppool, handler_ctx.response);
-  }
+  ret = apr_pstrdup(ppool, handler_ctx.response);
   if (set_headers_flag) {
     r->headers_out = apr_table_copy(pool, handler_ctx.headers_out);
     *response_len = handler_ctx.response_len;
@@ -497,16 +457,6 @@ default_chxj_serf_post(request_rec *r, apr_pool_t *ppool, const char *url_path,
       chxj_set_content_type(r, apr_pstrdup(r->pool, contentType));
     }
   }
-  {
-    int ii;
-    apr_array_header_t  *headers;
-    apr_table_entry_t   *hentryp;
-    headers = (apr_array_header_t*)apr_table_elts(r->headers_out);
-    hentryp = (apr_table_entry_t*)headers->elts;
-    for (ii=0; ii<headers->nelts; ii++) {
-      DBG(r, "key:[%s] val:[%s]", hentryp[ii].key, hentryp[ii].val);
-    }
-  }
   DBG(r, "end chxj_serf_post()");
   return ret;
 }
